@@ -1,6 +1,7 @@
 package images
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"strings"
@@ -9,9 +10,10 @@ import (
 	"github.com/containers/common/pkg/completion"
 	"github.com/containers/common/pkg/report"
 	"github.com/containers/image/v5/types"
+	"github.com/containers/podman/v4/cmd/podman/common"
 	"github.com/containers/podman/v4/cmd/podman/registry"
 	"github.com/containers/podman/v4/pkg/domain/entities"
-	"github.com/pkg/errors"
+	"github.com/containers/podman/v4/pkg/util"
 	"github.com/spf13/cobra"
 )
 
@@ -20,9 +22,11 @@ import (
 type searchOptionsWrapper struct {
 	entities.ImageSearchOptions
 	// CLI only flags
-	Compatible   bool   // Docker compat
-	TLSVerifyCLI bool   // Used to convert to an optional bool later
-	Format       string // For go templating
+	Compatible     bool // Docker compat
+	CredentialsCLI string
+	TLSVerifyCLI   bool   // Used to convert to an optional bool later
+	Format         string // For go templating
+	NoTrunc        bool
 }
 
 // listEntryTag is a utility structure used for json serialization.
@@ -81,40 +85,49 @@ func searchFlags(cmd *cobra.Command) {
 
 	filterFlagName := "filter"
 	flags.StringSliceVarP(&searchOptions.Filters, filterFlagName, "f", []string{}, "Filter output based on conditions provided (default [])")
-	// TODO add custom filter function
-	_ = cmd.RegisterFlagCompletionFunc(filterFlagName, completion.AutocompleteNone)
+	_ = cmd.RegisterFlagCompletionFunc(filterFlagName, common.AutocompleteImageSearchFilters)
 
 	formatFlagName := "format"
 	flags.StringVar(&searchOptions.Format, formatFlagName, "", "Change the output format to JSON or a Go template")
-	_ = cmd.RegisterFlagCompletionFunc(formatFlagName, completion.AutocompleteNone)
+	_ = cmd.RegisterFlagCompletionFunc(formatFlagName, common.AutocompleteFormat(&entities.ImageSearchReport{}))
 
 	limitFlagName := "limit"
 	flags.IntVar(&searchOptions.Limit, limitFlagName, 0, "Limit the number of results")
 	_ = cmd.RegisterFlagCompletionFunc(limitFlagName, completion.AutocompleteNone)
 
-	flags.Bool("no-trunc", true, "Do not truncate the output. Default: true")
+	flags.BoolVar(&searchOptions.NoTrunc, "no-trunc", false, "Do not truncate the output")
 	flags.BoolVar(&searchOptions.Compatible, "compatible", false, "List stars, official and automated columns (Docker compatibility)")
 
 	authfileFlagName := "authfile"
 	flags.StringVar(&searchOptions.Authfile, authfileFlagName, auth.GetDefaultAuthFile(), "Path of the authentication file. Use REGISTRY_AUTH_FILE environment variable to override")
 	_ = cmd.RegisterFlagCompletionFunc(authfileFlagName, completion.AutocompleteDefault)
 
+	credsFlagName := "creds"
+	flags.StringVar(&searchOptions.CredentialsCLI, credsFlagName, "", "`Credentials` (USERNAME:PASSWORD) to use for authenticating to a registry")
+	_ = cmd.RegisterFlagCompletionFunc(credsFlagName, completion.AutocompleteNone)
+
 	flags.BoolVar(&searchOptions.TLSVerifyCLI, "tls-verify", true, "Require HTTPS and verify certificates when contacting registries")
 	flags.BoolVar(&searchOptions.ListTags, "list-tags", false, "List the tags of the input registry")
+
+	if !registry.IsRemote() {
+		certDirFlagName := "cert-dir"
+		flags.StringVar(&searchOptions.CertDir, certDirFlagName, "", "`Pathname` of a directory containing TLS certificates and keys")
+		_ = cmd.RegisterFlagCompletionFunc(certDirFlagName, completion.AutocompleteDefault)
+	}
 }
 
 // imageSearch implements the command for searching images.
 func imageSearch(cmd *cobra.Command, args []string) error {
-	searchTerm := ""
+	var searchTerm string
 	switch len(args) {
 	case 1:
 		searchTerm = args[0]
 	default:
-		return errors.Errorf("search requires exactly one argument")
+		return errors.New("search requires exactly one argument")
 	}
 
 	if searchOptions.ListTags && len(searchOptions.Filters) != 0 {
-		return errors.Errorf("filters are not applicable to list tags result")
+		return errors.New("filters are not applicable to list tags result")
 	}
 
 	// TLS verification in c/image is controlled via a `types.OptionalBool`
@@ -131,6 +144,15 @@ func imageSearch(cmd *cobra.Command, args []string) error {
 		}
 	}
 
+	if searchOptions.CredentialsCLI != "" {
+		creds, err := util.ParseRegistryCreds(searchOptions.CredentialsCLI)
+		if err != nil {
+			return err
+		}
+		searchOptions.Username = creds.Username
+		searchOptions.Password = creds.Password
+	}
+
 	searchReport, err := registry.ImageEngine().Search(registry.GetContext(), searchTerm, searchOptions.ImageSearchOptions)
 	if err != nil {
 		return err
@@ -139,11 +161,10 @@ func imageSearch(cmd *cobra.Command, args []string) error {
 		return nil
 	}
 
-	noTrunc, _ := cmd.Flags().GetBool("no-trunc")
 	isJSON := report.IsJSON(searchOptions.Format)
 	for i, element := range searchReport {
 		d := strings.ReplaceAll(element.Description, "\n", " ")
-		if len(d) > 44 && !(noTrunc || isJSON) {
+		if len(d) > 44 && !(searchOptions.NoTrunc || isJSON) {
 			d = strings.TrimSpace(d[:44]) + "..."
 		}
 		searchReport[i].Description = d
@@ -155,7 +176,7 @@ func imageSearch(cmd *cobra.Command, args []string) error {
 	switch {
 	case searchOptions.ListTags:
 		if len(searchOptions.Filters) != 0 {
-			return errors.Errorf("filters are not applicable to list tags result")
+			return errors.New("filters are not applicable to list tags result")
 		}
 		if isJSON {
 			listTagsEntries := buildListTagsJSON(searchReport)
@@ -181,7 +202,7 @@ func imageSearch(cmd *cobra.Command, args []string) error {
 	if rpt.RenderHeaders {
 		hdrs := report.Headers(entities.ImageSearchReport{}, nil)
 		if err := rpt.Execute(hdrs); err != nil {
-			return errors.Wrapf(err, "failed to write report column headers")
+			return fmt.Errorf("failed to write report column headers: %w", err)
 		}
 	}
 	return rpt.Execute(searchReport)
