@@ -13,6 +13,7 @@ import (
 	"os/user"
 	"path"
 	"path/filepath"
+	"runtime"
 	"strconv"
 	"strings"
 	"syscall"
@@ -57,11 +58,17 @@ import (
 	"github.com/sirupsen/logrus"
 )
 
-func parseOptionIDs(option string) ([]idtools.IDMap, error) {
+func parseOptionIDs(ctrMappings []idtools.IDMap, option string) ([]idtools.IDMap, error) {
 	ranges := strings.Split(option, "#")
 	ret := make([]idtools.IDMap, len(ranges))
 	for i, m := range ranges {
 		var v idtools.IDMap
+
+		relative := false
+		if m[0] == '@' {
+			relative = true
+			m = m[1:]
+		}
 		_, err := fmt.Sscanf(m, "%d-%d-%d", &v.ContainerID, &v.HostID, &v.Size)
 		if err != nil {
 			return nil, err
@@ -69,6 +76,20 @@ func parseOptionIDs(option string) ([]idtools.IDMap, error) {
 		if v.ContainerID < 0 || v.HostID < 0 || v.Size < 1 {
 			return nil, fmt.Errorf("invalid value for %q", option)
 		}
+
+		if relative {
+			found := false
+			for _, m := range ctrMappings {
+				if v.ContainerID >= m.ContainerID && v.ContainerID < m.ContainerID+m.Size {
+					v.HostID += m.HostID - m.ContainerID
+					found = true
+					break
+				}
+			}
+			if !found {
+				return nil, fmt.Errorf("could not find a user namespace mapping for the relative mapping %q", option)
+			}
+		}
 		ret[i] = v
 	}
 	return ret, nil
@@ -83,12 +104,12 @@ func parseIDMapMountOption(idMappings stypes.IDMappingOptions, option string) ([
 		for _, i := range options {
 			switch {
 			case strings.HasPrefix(i, "uids="):
-				uidMap, err = parseOptionIDs(strings.Replace(i, "uids=", "", 1))
+				uidMap, err = parseOptionIDs(idMappings.UIDMap, strings.Replace(i, "uids=", "", 1))
 				if err != nil {
 					return nil, nil, err
 				}
 			case strings.HasPrefix(i, "gids="):
-				gidMap, err = parseOptionIDs(strings.Replace(i, "gids=", "", 1))
+				gidMap, err = parseOptionIDs(idMappings.GIDMap, strings.Replace(i, "gids=", "", 1))
 				if err != nil {
 					return nil, nil, err
 				}
@@ -150,7 +171,21 @@ func getOverlayUpperAndWorkDir(options []string) (string, string, error) {
 
 // Generate spec for a container
 // Accepts a map of the container's dependencies
-func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
+func (c *Container) generateSpec(ctx context.Context) (s *spec.Spec, cleanupFuncRet func(), err error) {
+	var safeMounts []*safeMountInfo
+	// lock the thread so that the current thread will be kept alive until the mounts are used
+	runtime.LockOSThread()
+	cleanupFunc := func() {
+		runtime.UnlockOSThread()
+		for _, s := range safeMounts {
+			s.Close()
+		}
+	}
+	defer func() {
+		if err != nil {
+			cleanupFunc()
+		}
+	}()
 	overrides := c.getUserOverrides()
 	execUser, err := lookup.GetUserGroupInfo(c.state.Mountpoint, c.config.User, overrides)
 	if err != nil {
@@ -158,13 +193,12 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 			execUser, err = lookupHostUser(c.config.User)
 		}
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 	}
 
 	// NewFromSpec() is deprecated according to its comment
 	// however the recommended replace just causes a nil map panic
-	//nolint:staticcheck
 	g := generate.NewFromSpec(c.config.Spec)
 
 	// If the flag to mount all devices is set for a privileged container, add
@@ -175,51 +209,57 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 			systemdMode = *c.config.Systemd
 		}
 		if err := util.AddPrivilegedDevices(&g, systemdMode); err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 	}
 
 	// If network namespace was requested, add it now
 	if err := c.addNetworkNamespace(&g); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Apply AppArmor checks and load the default profile if needed.
 	if len(c.config.Spec.Process.ApparmorProfile) > 0 {
 		updatedProfile, err := apparmor.CheckProfileAndLoadDefault(c.config.Spec.Process.ApparmorProfile)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		g.SetProcessApparmorProfile(updatedProfile)
 	}
 
 	if err := c.makeBindMounts(); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	if err := c.mountNotifySocket(g); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Get host UID and GID based on the container process UID and GID.
 	hostUID, hostGID, err := butil.GetHostIDs(util.IDtoolsToRuntimeSpec(c.config.IDMappings.UIDMap), util.IDtoolsToRuntimeSpec(c.config.IDMappings.GIDMap), uint32(execUser.Uid), uint32(execUser.Gid))
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Add named volumes
 	for _, namedVol := range c.config.NamedVolumes {
 		volume, err := c.runtime.GetVolume(namedVol.Name)
 		if err != nil {
-			return nil, fmt.Errorf("retrieving volume %s to add to container %s: %w", namedVol.Name, c.ID(), err)
+			return nil, nil, fmt.Errorf("retrieving volume %s to add to container %s: %w", namedVol.Name, c.ID(), err)
 		}
 		mountPoint, err := volume.MountPoint()
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 
 		if len(namedVol.SubPath) > 0 {
-			mountPoint = filepath.Join(mountPoint, namedVol.SubPath)
+			safeMount, err := c.safeMountSubPath(mountPoint, namedVol.SubPath)
+			if err != nil {
+				return nil, nil, err
+			}
+			safeMounts = append(safeMounts, safeMount)
+
+			mountPoint = safeMount.mountPoint
 		}
 
 		overlayFlag := false
@@ -230,7 +270,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 				overlayFlag = true
 				upperDir, workDir, err = getOverlayUpperAndWorkDir(namedVol.Options)
 				if err != nil {
-					return nil, err
+					return nil, nil, err
 				}
 			}
 		}
@@ -240,7 +280,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 			var overlayOpts *overlay.Options
 			contentDir, err := overlay.TempDir(c.config.StaticDir, c.RootUID(), c.RootGID())
 			if err != nil {
-				return nil, err
+				return nil, nil, err
 			}
 
 			overlayOpts = &overlay.Options{RootUID: c.RootUID(),
@@ -252,17 +292,17 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 
 			overlayMount, err = overlay.MountWithOptions(contentDir, mountPoint, namedVol.Dest, overlayOpts)
 			if err != nil {
-				return nil, fmt.Errorf("mounting overlay failed %q: %w", mountPoint, err)
+				return nil, nil, fmt.Errorf("mounting overlay failed %q: %w", mountPoint, err)
 			}
 
 			for _, o := range namedVol.Options {
 				if o == "U" {
 					if err := c.ChangeHostPathOwnership(mountPoint, true, int(hostUID), int(hostGID)); err != nil {
-						return nil, err
+						return nil, nil, err
 					}
 
 					if err := c.ChangeHostPathOwnership(contentDir, true, int(hostUID), int(hostGID)); err != nil {
-						return nil, err
+						return nil, nil, err
 					}
 				}
 			}
@@ -280,17 +320,27 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 
 	// Check if the spec file mounts contain the options z, Z, U or idmap.
 	// If they have z or Z, relabel the source directory and then remove the option.
-	// If they have U, chown the source directory and them remove the option.
+	// If they have U, chown the source directory and then remove the option.
 	// If they have idmap, then calculate the mappings to use in the OCI config file.
 	for i := range g.Config.Mounts {
 		m := &g.Config.Mounts[i]
 		var options []string
 		for _, o := range m.Options {
+			if strings.HasPrefix(o, "subpath=") {
+				subpath := strings.Split(o, "=")[1]
+				safeMount, err := c.safeMountSubPath(m.Source, subpath)
+				if err != nil {
+					return nil, nil, err
+				}
+				safeMounts = append(safeMounts, safeMount)
+				m.Source = safeMount.mountPoint
+				continue
+			}
 			if o == "idmap" || strings.HasPrefix(o, "idmap=") {
 				var err error
 				m.UIDMappings, m.GIDMappings, err = parseIDMapMountOption(c.config.IDMappings, o)
 				if err != nil {
-					return nil, err
+					return nil, nil, err
 				}
 				continue
 			}
@@ -301,14 +351,14 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 				} else {
 					// only chown on initial creation of container
 					if err := c.ChangeHostPathOwnership(m.Source, true, int(hostUID), int(hostGID)); err != nil {
-						return nil, err
+						return nil, nil, err
 					}
 				}
 			case "z":
 				fallthrough
 			case "Z":
 				if err := c.relabel(m.Source, c.MountLabel(), label.IsShared(o)); err != nil {
-					return nil, err
+					return nil, nil, err
 				}
 
 			default:
@@ -346,11 +396,11 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	for _, overlayVol := range c.config.OverlayVolumes {
 		upperDir, workDir, err := getOverlayUpperAndWorkDir(overlayVol.Options)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		contentDir, err := overlay.TempDir(c.config.StaticDir, c.RootUID(), c.RootGID())
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		overlayOpts := &overlay.Options{RootUID: c.RootUID(),
 			RootGID:                c.RootGID(),
@@ -361,18 +411,18 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 
 		overlayMount, err := overlay.MountWithOptions(contentDir, overlayVol.Source, overlayVol.Dest, overlayOpts)
 		if err != nil {
-			return nil, fmt.Errorf("mounting overlay failed %q: %w", overlayVol.Source, err)
+			return nil, nil, fmt.Errorf("mounting overlay failed %q: %w", overlayVol.Source, err)
 		}
 
 		// Check overlay volume options
 		for _, o := range overlayVol.Options {
 			if o == "U" {
 				if err := c.ChangeHostPathOwnership(overlayVol.Source, true, int(hostUID), int(hostGID)); err != nil {
-					return nil, err
+					return nil, nil, err
 				}
 
 				if err := c.ChangeHostPathOwnership(contentDir, true, int(hostUID), int(hostGID)); err != nil {
-					return nil, err
+					return nil, nil, err
 				}
 			}
 		}
@@ -385,16 +435,16 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 		// Mount the specified image.
 		img, _, err := c.runtime.LibimageRuntime().LookupImage(volume.Source, nil)
 		if err != nil {
-			return nil, fmt.Errorf("creating image volume %q:%q: %w", volume.Source, volume.Dest, err)
+			return nil, nil, fmt.Errorf("creating image volume %q:%q: %w", volume.Source, volume.Dest, err)
 		}
 		mountPoint, err := img.Mount(ctx, nil, "")
 		if err != nil {
-			return nil, fmt.Errorf("mounting image volume %q:%q: %w", volume.Source, volume.Dest, err)
+			return nil, nil, fmt.Errorf("mounting image volume %q:%q: %w", volume.Source, volume.Dest, err)
 		}
 
 		contentDir, err := overlay.TempDir(c.config.StaticDir, c.RootUID(), c.RootGID())
 		if err != nil {
-			return nil, fmt.Errorf("failed to create TempDir in the %s directory: %w", c.config.StaticDir, err)
+			return nil, nil, fmt.Errorf("failed to create TempDir in the %s directory: %w", c.config.StaticDir, err)
 		}
 
 		var overlayMount spec.Mount
@@ -404,7 +454,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 			overlayMount, err = overlay.MountReadOnly(contentDir, mountPoint, volume.Dest, c.RootUID(), c.RootGID(), c.runtime.store.GraphOptions())
 		}
 		if err != nil {
-			return nil, fmt.Errorf("creating overlay mount for image %q failed: %w", volume.Source, err)
+			return nil, nil, fmt.Errorf("creating overlay mount for image %q failed: %w", volume.Source, err)
 		}
 		g.AddMount(overlayMount)
 	}
@@ -430,7 +480,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	if c.config.Umask != "" {
 		decVal, err := strconv.ParseUint(c.config.Umask, 8, 32)
 		if err != nil {
-			return nil, fmt.Errorf("invalid Umask Value: %w", err)
+			return nil, nil, fmt.Errorf("invalid Umask Value: %w", err)
 		}
 		umask := uint32(decVal)
 		g.Config.Process.User.Umask = &umask
@@ -440,7 +490,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	if len(c.config.Groups) > 0 {
 		gids, err := lookup.GetContainerGroups(c.config.Groups, c.state.Mountpoint, overrides)
 		if err != nil {
-			return nil, fmt.Errorf("looking up supplemental groups for container %s: %w", c.ID(), err)
+			return nil, nil, fmt.Errorf("looking up supplemental groups for container %s: %w", c.ID(), err)
 		}
 		for _, gid := range gids {
 			g.AddProcessAdditionalGid(gid)
@@ -448,7 +498,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	}
 
 	if err := c.addSystemdMounts(&g); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Look up and add groups the user belongs to, if a group wasn't directly specified
@@ -463,7 +513,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 			// Check whether the current user namespace has enough gids available.
 			availableGids, err := rootless.GetAvailableGids()
 			if err != nil {
-				return nil, fmt.Errorf("cannot read number of available GIDs: %w", err)
+				return nil, nil, fmt.Errorf("cannot read number of available GIDs: %w", err)
 			}
 			gidMappings = []idtools.IDMap{{
 				ContainerID: 0,
@@ -495,7 +545,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 
 	// Add shared namespaces from other containers
 	if err := c.addSharedNamespaces(&g); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	g.SetRootPath(c.state.Mountpoint)
@@ -506,7 +556,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	}
 
 	if err := c.setCgroupsPath(&g); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Warning: CDI may alter g.Config in place.
@@ -519,7 +569,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 		}
 		_, err := registry.InjectDevices(g.Config, c.config.CDIDevices...)
 		if err != nil {
-			return nil, fmt.Errorf("setting up CDI devices: %w", err)
+			return nil, nil, fmt.Errorf("setting up CDI devices: %w", err)
 		}
 	}
 
@@ -535,7 +585,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 		if m.Type == "tmpfs" {
 			finalPath, err := securejoin.SecureJoin(c.state.Mountpoint, m.Destination)
 			if err != nil {
-				return nil, fmt.Errorf("resolving symlinks for mount destination %s: %w", m.Destination, err)
+				return nil, nil, fmt.Errorf("resolving symlinks for mount destination %s: %w", m.Destination, err)
 			}
 			trimmedPath := strings.TrimPrefix(finalPath, strings.TrimSuffix(c.state.Mountpoint, "/"))
 			m.Destination = trimmedPath
@@ -544,25 +594,25 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 	}
 
 	if err := c.addRootPropagation(&g, mounts); err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 
 	// Warning: precreate hooks may alter g.Config in place.
 	if c.state.ExtensionStageHooks, err = c.setupOCIHooks(ctx, g.Config); err != nil {
-		return nil, fmt.Errorf("setting up OCI Hooks: %w", err)
+		return nil, nil, fmt.Errorf("setting up OCI Hooks: %w", err)
 	}
 	if len(c.config.EnvSecrets) > 0 {
 		manager, err := c.runtime.SecretsManager()
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		for name, secr := range c.config.EnvSecrets {
 			_, data, err := manager.LookupSecretData(secr.Name)
 			if err != nil {
-				return nil, err
+				return nil, nil, err
 			}
 			g.AddProcessEnv(name, string(data))
 		}
@@ -580,7 +630,7 @@ func (c *Container) generateSpec(ctx context.Context) (*spec.Spec, error) {
 		}
 	}
 
-	return g.Config, nil
+	return g.Config, cleanupFunc, nil
 }
 
 // isWorkDirSymlink returns true if resolved workdir is symlink or a chain of symlinks,
@@ -1205,7 +1255,6 @@ func (c *Container) generateContainerSpec() error {
 
 	// NewFromSpec() is deprecated according to its comment
 	// however the recommended replace just causes a nil map panic
-	//nolint:staticcheck
 	g := generate.NewFromSpec(c.config.Spec)
 
 	if err := c.saveSpec(g.Config); err != nil {
@@ -1315,7 +1364,7 @@ func (c *Container) restore(ctx context.Context, options ContainerCheckpointOpti
 	}
 
 	// Let's try to stat() CRIU's inventory file. If it does not exist, it makes
-	// no sense to try a restore. This is a minimal check if a checkpoint exist.
+	// no sense to try a restore. This is a minimal check if a checkpoint exists.
 	if _, err := os.Stat(filepath.Join(c.CheckpointPath(), "inventory.img")); os.IsNotExist(err) {
 		return nil, 0, fmt.Errorf("a complete checkpoint for this container cannot be found, cannot restore: %w", err)
 	}
@@ -2019,8 +2068,13 @@ func (c *Container) generateResolvConf() error {
 		}
 		// first add the nameservers from the networks status
 		nameservers = networkNameServers
+
 		// slirp4netns has a built in DNS forwarder.
-		nameservers = c.addSlirp4netnsDNS(nameservers)
+		// If in userns the network is not setup here, instead we need to do that in
+		// c.completeNetworkSetup() which knows the actual slirp dns ip only at that point
+		if !c.config.PostConfigureNetNS {
+			nameservers = c.addSlirp4netnsDNS(nameservers)
+		}
 	}
 
 	// Set DNS search domains
@@ -2214,7 +2268,7 @@ func (c *Container) generateGroupEntry() (string, error) {
 		groupString += entry
 		addedGID = gid
 	}
-	if c.config.User != "" {
+	if c.config.User != "" || c.config.GroupEntry != "" {
 		entry, err := c.generateUserGroupEntry(addedGID)
 		if err != nil {
 			return "", err
@@ -2269,7 +2323,7 @@ func (c *Container) generateCurrentUserGroupEntry() (string, int, error) {
 // Make an entry in /etc/group for the group the container was specified to run
 // as.
 func (c *Container) generateUserGroupEntry(addedGID int) (string, error) {
-	if c.config.User == "" {
+	if c.config.User == "" && c.config.GroupEntry == "" {
 		return "", nil
 	}
 
@@ -2289,14 +2343,26 @@ func (c *Container) generateUserGroupEntry(addedGID int) (string, error) {
 	}
 
 	// Check if the group already exists
-	_, err = lookup.GetGroup(c.state.Mountpoint, group)
+	g, err := lookup.GetGroup(c.state.Mountpoint, group)
 	if err != runcuser.ErrNoGroupEntries {
 		return "", err
 	}
 
+	if c.config.GroupEntry != "" {
+		return c.groupEntry(g.Name, strconv.Itoa(g.Gid), g.List), nil
+	}
+
 	return fmt.Sprintf("%d:x:%d:%s\n", gid, gid, splitUser[0]), nil
 }
 
+func (c *Container) groupEntry(groupname, gid string, list []string) string {
+	s := c.config.GroupEntry
+	s = strings.ReplaceAll(s, "$GROUPNAME", groupname)
+	s = strings.ReplaceAll(s, "$GID", gid)
+	s = strings.ReplaceAll(s, "$USERLIST", strings.Join(list, ","))
+	return s + "\n"
+}
+
 // generatePasswdEntry generates an entry or entries into /etc/passwd as
 // required by container configuration.
 // Generally speaking, we will make an entry under two circumstances:
@@ -2476,7 +2542,7 @@ func (c *Container) generateUserPasswdEntry(addedUID int) (string, error) {
 	return fmt.Sprintf("%d:*:%d:%d:container user:%s:/bin/sh\n", uid, uid, gid, c.WorkingDir()), nil
 }
 
-func (c *Container) passwdEntry(username string, uid, gid, name, homeDir string) string {
+func (c *Container) passwdEntry(username, uid, gid, name, homeDir string) string {
 	s := c.config.PasswdEntry
 	s = strings.ReplaceAll(s, "$USERNAME", username)
 	s = strings.ReplaceAll(s, "$UID", uid)
@@ -2500,7 +2566,7 @@ func (c *Container) passwdEntry(username string, uid, gid, name, homeDir string)
 // read-only. In this case, the function will return nothing ("", "", nil).
 func (c *Container) generatePasswdAndGroup() (string, string, error) {
 	if !c.config.AddCurrentUserPasswdEntry && c.config.User == "" &&
-		len(c.config.HostUsers) == 0 {
+		len(c.config.HostUsers) == 0 && c.config.GroupEntry == "" {
 		return "", "", nil
 	}
 
@@ -2715,7 +2781,7 @@ func (c *Container) fixVolumePermissions(v *ContainerNamedVolume) error {
 
 	// Volumes owned by a volume driver are not chowned - we don't want to
 	// mess with a mount not managed by us.
-	if vol.state.NeedsChown && !vol.UsesVolumeDriver() {
+	if vol.state.NeedsChown && (!vol.UsesVolumeDriver() && vol.config.Driver != "image") {
 		vol.state.NeedsChown = false
 
 		uid := int(c.config.Spec.Process.User.UID)
