@@ -5,12 +5,14 @@ import (
 	"fmt"
 
 	"github.com/containers/common/libimage"
+	"github.com/containers/podman/v4/libpod/define"
 	"github.com/containers/podman/v4/pkg/domain/entities"
 )
 
 func (ir *ImageEngine) List(ctx context.Context, opts entities.ImageListOptions) ([]*entities.ImageSummary, error) {
 	listImagesOptions := &libimage.ListImagesOptions{
-		Filters: opts.Filter,
+		Filters:     opts.Filter,
+		SetListData: true,
 	}
 	if !opts.All {
 		// Filter intermediate images unless we want to list *all*.
@@ -30,9 +32,14 @@ func (ir *ImageEngine) List(ctx context.Context, opts entities.ImageListOptions)
 		if err != nil {
 			return nil, fmt.Errorf("getting repoDigests from image %q: %w", img.ID(), err)
 		}
-		isDangling, err := img.IsDangling(ctx)
-		if err != nil {
-			return nil, fmt.Errorf("checking if image %q is dangling: %w", img.ID(), err)
+
+		if img.ListData.IsDangling == nil { // Sanity check
+			return nil, fmt.Errorf("%w: ListData.IsDangling is nil but should not", define.ErrInternal)
+		}
+		isDangling := *img.ListData.IsDangling
+		parentID := ""
+		if img.ListData.Parent != nil {
+			parentID = img.ListData.Parent.ID()
 		}
 
 		e := entities.ImageSummary{
@@ -46,6 +53,7 @@ func (ir *ImageEngine) List(ctx context.Context, opts entities.ImageListOptions)
 			ReadOnly:    img.IsReadOnly(),
 			SharedSize:  0,
 			RepoTags:    img.Names(), // may include tags and digests
+			ParentId:    parentID,
 		}
 		e.Labels, err = img.Labels(ctx)
 		if err != nil {
@@ -67,14 +75,6 @@ func (ir *ImageEngine) List(ctx context.Context, opts entities.ImageListOptions)
 		// replaced later with correct calculation logic
 		e.VirtualSize = sz
 
-		parent, err := img.Parent(ctx)
-		if err != nil {
-			return nil, fmt.Errorf("retrieving parent of image %q: you may need to remove the image to resolve the error: %w", img.ID(), err)
-		}
-		if parent != nil {
-			e.ParentId = parent.ID()
-		}
-
 		summaries = append(summaries, &e)
 	}
 	return summaries, nil
