@@ -7,7 +7,6 @@ import (
 	"time"
 
 	"github.com/containers/common/libnetwork/types"
-	"github.com/containers/podman/v4/pkg/rootless"
 	. "github.com/containers/podman/v4/test/utils"
 	"github.com/containers/storage/pkg/stringid"
 	. "github.com/onsi/ginkgo"
@@ -48,14 +47,6 @@ var _ = Describe("Podman network", func() {
 		Expect(session).Should(Exit(0))
 		// default network always exists
 		Expect(session.OutputToStringArray()).To(HaveLen(1))
-
-		// check that the only file in the directory is the network lockfile
-		dir, err := os.Open(netDir)
-		Expect(err).ToNot(HaveOccurred())
-		names, err := dir.Readdirnames(5)
-		Expect(err).ToNot(HaveOccurred())
-		Expect(names).To(HaveLen(1))
-		Expect(names[0]).To(Or(Equal("netavark.lock"), Equal("cni.lock")))
 	})
 
 	It("podman network list", func() {
@@ -113,7 +104,7 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network list --filter labels", func() {
-		net1 := "labelnet" + stringid.GenerateNonCryptoID()
+		net1 := "labelnet" + stringid.GenerateRandomID()
 		label1 := "testlabel1=abc"
 		label2 := "abcdef"
 		session := podmanTest.Podman([]string{"network", "create", "--label", label1, net1})
@@ -121,7 +112,7 @@ var _ = Describe("Podman network", func() {
 		defer podmanTest.removeNetwork(net1)
 		Expect(session).Should(Exit(0))
 
-		net2 := "labelnet" + stringid.GenerateNonCryptoID()
+		net2 := "labelnet" + stringid.GenerateRandomID()
 		session = podmanTest.Podman([]string{"network", "create", "--label", label1, "--label", label2, net2})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net2)
@@ -141,7 +132,7 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network list --filter invalid value", func() {
-		net := "net" + stringid.GenerateNonCryptoID()
+		net := "net" + stringid.GenerateRandomID()
 		session := podmanTest.Podman([]string{"network", "create", net})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
@@ -163,6 +154,26 @@ var _ = Describe("Podman network", func() {
 		Expect(session.OutputToString()).To(Not(ContainSubstring(name)))
 	})
 
+	It("podman network list --filter dangling", func() {
+		name, path := generateNetworkConfig(podmanTest)
+		defer removeConf(path)
+
+		session := podmanTest.Podman([]string{"network", "ls", "--filter", "dangling=true"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).To(ContainSubstring(name))
+
+		session = podmanTest.Podman([]string{"network", "ls", "--filter", "dangling=false"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(Exit(0))
+		Expect(session.OutputToString()).NotTo(ContainSubstring(name))
+
+		session = podmanTest.Podman([]string{"network", "ls", "--filter", "dangling=foo"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).To(ExitWithError())
+		Expect(session.ErrorToString()).To(ContainSubstring(`invalid dangling filter value "foo"`))
+	})
+
 	It("podman network ID test", func() {
 		net := "networkIDTest"
 		// the network id should be the sha256 hash of the network name
@@ -253,7 +264,7 @@ var _ = Describe("Podman network", func() {
 		defer removeConf(path)
 
 		expectedNetworks := []string{name}
-		if !rootless.IsRootless() {
+		if !isRootless() {
 			// rootful image contains "podman/cni/87-podman-bridge.conflist" for "podman" network
 			expectedNetworks = append(expectedNetworks, "podman")
 		}
@@ -274,7 +285,7 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman inspect container single CNI network", func() {
-		netName := "net-" + stringid.GenerateNonCryptoID()
+		netName := "net-" + stringid.GenerateRandomID()
 		network := podmanTest.Podman([]string{"network", "create", "--subnet", "10.50.50.0/24", netName})
 		network.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName)
@@ -304,13 +315,13 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman inspect container two CNI networks (container not running)", func() {
-		netName1 := "net1-" + stringid.GenerateNonCryptoID()
+		netName1 := "net1-" + stringid.GenerateRandomID()
 		network1 := podmanTest.Podman([]string{"network", "create", netName1})
 		network1.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName1)
 		Expect(network1).Should(Exit(0))
 
-		netName2 := "net2-" + stringid.GenerateNonCryptoID()
+		netName2 := "net2-" + stringid.GenerateRandomID()
 		network2 := podmanTest.Podman([]string{"network", "create", netName2})
 		network2.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName2)
@@ -341,13 +352,13 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman inspect container two CNI networks", func() {
-		netName1 := "net1-" + stringid.GenerateNonCryptoID()
+		netName1 := "net1-" + stringid.GenerateRandomID()
 		network1 := podmanTest.Podman([]string{"network", "create", "--subnet", "10.50.51.0/25", netName1})
 		network1.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName1)
 		Expect(network1).Should(Exit(0))
 
-		netName2 := "net2-" + stringid.GenerateNonCryptoID()
+		netName2 := "net2-" + stringid.GenerateRandomID()
 		network2 := podmanTest.Podman([]string{"network", "create", "--subnet", "10.50.51.128/26", netName2})
 		network2.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName2)
@@ -383,7 +394,7 @@ var _ = Describe("Podman network", func() {
 
 	It("podman network remove after disconnect when container initially created with the network", func() {
 		container := "test"
-		network := "foo" + stringid.GenerateNonCryptoID()
+		network := "foo" + stringid.GenerateRandomID()
 
 		session := podmanTest.Podman([]string{"network", "create", network})
 		session.WaitWithDefaultTimeout()
@@ -410,7 +421,7 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network remove --force with pod", func() {
-		netName := "net-" + stringid.GenerateNonCryptoID()
+		netName := "net-" + stringid.GenerateRandomID()
 		session := podmanTest.Podman([]string{"network", "create", netName})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName)
@@ -446,13 +457,13 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network remove with two networks", func() {
-		netName1 := "net1-" + stringid.GenerateNonCryptoID()
+		netName1 := "net1-" + stringid.GenerateRandomID()
 		session := podmanTest.Podman([]string{"network", "create", netName1})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName1)
 		Expect(session).Should(Exit(0))
 
-		netName2 := "net2-" + stringid.GenerateNonCryptoID()
+		netName2 := "net2-" + stringid.GenerateRandomID()
 		session = podmanTest.Podman([]string{"network", "create", netName2})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(netName2)
@@ -466,8 +477,7 @@ var _ = Describe("Podman network", func() {
 		Expect(lines[1]).To(Equal(netName2))
 	})
 
-	It("podman CNI network with multiple aliases", func() {
-		SkipIfNetavark(podmanTest)
+	It("podman network with multiple aliases", func() {
 		var worked bool
 		netName := createNetworkName("aliasTest")
 		session := podmanTest.Podman([]string{"network", "create", netName})
@@ -475,7 +485,7 @@ var _ = Describe("Podman network", func() {
 		defer podmanTest.removeNetwork(netName)
 		Expect(session).Should(Exit(0))
 
-		interval := time.Duration(250 * time.Millisecond)
+		interval := 250 * time.Millisecond
 		for i := 0; i < 6; i++ {
 			n := podmanTest.Podman([]string{"network", "exists", netName})
 			n.WaitWithDefaultTimeout()
@@ -487,14 +497,14 @@ var _ = Describe("Podman network", func() {
 			interval *= 2
 		}
 
-		top := podmanTest.Podman([]string{"run", "-dt", "--name=web", "--network=" + netName, "--network-alias=web1", "--network-alias=web2", nginx})
+		top := podmanTest.Podman([]string{"run", "-dt", "--name=web", "--network=" + netName, "--network-alias=web1", "--network-alias=web2", NGINX_IMAGE})
 		top.WaitWithDefaultTimeout()
 		Expect(top).Should(Exit(0))
-		interval = time.Duration(250 * time.Millisecond)
+		interval = 250 * time.Millisecond
 		// Wait for the nginx service to be running
 		for i := 0; i < 6; i++ {
 			// Test curl against the container's name
-			c1 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web"})
+			c1 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, NGINX_IMAGE, "curl", "web"})
 			c1.WaitWithDefaultTimeout()
 			worked = c1.ExitCode() == 0
 			if worked {
@@ -507,63 +517,12 @@ var _ = Describe("Podman network", func() {
 
 		// Nginx is now running so no need to do a loop
 		// Test against the first alias
-		c2 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web1"})
+		c2 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, NGINX_IMAGE, "curl", "web1"})
 		c2.WaitWithDefaultTimeout()
 		Expect(c2).Should(Exit(0))
 
 		// Test against the second alias
-		c3 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web2"})
-		c3.WaitWithDefaultTimeout()
-		Expect(c3).Should(Exit(0))
-	})
-
-	It("podman Netavark network with multiple aliases", func() {
-		SkipIfCNI(podmanTest)
-		var worked bool
-		netName := createNetworkName("aliasTest")
-		session := podmanTest.Podman([]string{"network", "create", netName})
-		session.WaitWithDefaultTimeout()
-		defer podmanTest.removeNetwork(netName)
-		Expect(session).Should(Exit(0))
-
-		interval := time.Duration(250 * time.Millisecond)
-		for i := 0; i < 6; i++ {
-			n := podmanTest.Podman([]string{"network", "exists", netName})
-			n.WaitWithDefaultTimeout()
-			worked = n.ExitCode() == 0
-			if worked {
-				break
-			}
-			time.Sleep(interval)
-			interval *= 2
-		}
-
-		top := podmanTest.Podman([]string{"run", "-dt", "--name=web", "--network=" + netName, "--network-alias=web1", "--network-alias=web2", nginx})
-		top.WaitWithDefaultTimeout()
-		Expect(top).Should(Exit(0))
-		interval = time.Duration(250 * time.Millisecond)
-		// Wait for the nginx service to be running
-		for i := 0; i < 6; i++ {
-			// Test curl against the container's name
-			c1 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web"})
-			c1.WaitWithDefaultTimeout()
-			worked = c1.ExitCode() == 0
-			if worked {
-				break
-			}
-			time.Sleep(interval)
-			interval *= 2
-		}
-		Expect(worked).To(BeTrue())
-
-		// Nginx is now running so no need to do a loop
-		// Test against the first alias
-		c2 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web1"})
-		c2.WaitWithDefaultTimeout()
-		Expect(c2).Should(Exit(0))
-
-		// Test against the second alias
-		c3 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, nginx, "curl", "web2"})
+		c3 := podmanTest.Podman([]string{"run", "--dns-search", "dns.podman", "--network=" + netName, NGINX_IMAGE, "curl", "web2"})
 		c3.WaitWithDefaultTimeout()
 		Expect(c3).Should(Exit(0))
 	})
@@ -571,7 +530,7 @@ var _ = Describe("Podman network", func() {
 	It("podman network create/remove macvlan", func() {
 		// Netavark currently does not do dhcp so the this test fails
 		SkipIfNetavark(podmanTest)
-		net := "macvlan" + stringid.GenerateNonCryptoID()
+		net := "macvlan" + stringid.GenerateRandomID()
 		nc := podmanTest.Podman([]string{"network", "create", "--macvlan", "lo", net})
 		nc.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
@@ -585,7 +544,7 @@ var _ = Describe("Podman network", func() {
 	It("podman network create/remove macvlan as driver (-d) no device name", func() {
 		// Netavark currently does not do dhcp so the this test fails
 		SkipIfNetavark(podmanTest)
-		net := "macvlan" + stringid.GenerateNonCryptoID()
+		net := "macvlan" + stringid.GenerateRandomID()
 		nc := podmanTest.Podman([]string{"network", "create", "-d", "macvlan", net})
 		nc.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
@@ -598,7 +557,7 @@ var _ = Describe("Podman network", func() {
 		// JSON the network configuration into something usable
 		var results []types.Network
 		err := json.Unmarshal([]byte(inspect.OutputToString()), &results)
-		Expect(err).To(BeNil())
+		Expect(err).ToNot(HaveOccurred())
 		Expect(results).To(HaveLen(1))
 		result := results[0]
 		Expect(result).To(HaveField("NetworkInterface", ""))
@@ -609,11 +568,43 @@ var _ = Describe("Podman network", func() {
 		Expect(nc).Should(Exit(0))
 	})
 
-	It("podman network create/remove macvlan as driver (-d) with device name", func() {
-		// Netavark currently does not do dhcp so the this test fails
+	for _, opt := range []string{"-o=parent=lo", "--interface-name=lo"} {
+		opt := opt
+		It(fmt.Sprintf("podman network create/remove macvlan as driver (-d) with %s", opt), func() {
+			// Netavark currently does not do dhcp so the this test fails
+			SkipIfNetavark(podmanTest)
+			net := "macvlan" + stringid.GenerateRandomID()
+			nc := podmanTest.Podman([]string{"network", "create", "-d", "macvlan", opt, net})
+			nc.WaitWithDefaultTimeout()
+			defer podmanTest.removeNetwork(net)
+			Expect(nc).Should(Exit(0))
+
+			inspect := podmanTest.Podman([]string{"network", "inspect", net})
+			inspect.WaitWithDefaultTimeout()
+			Expect(inspect).Should(Exit(0))
+
+			var results []types.Network
+			err := json.Unmarshal([]byte(inspect.OutputToString()), &results)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(results).To(HaveLen(1))
+			result := results[0]
+
+			Expect(result).To(HaveField("Driver", "macvlan"))
+			Expect(result).To(HaveField("NetworkInterface", "lo"))
+			Expect(result.IPAMOptions).To(HaveKeyWithValue("driver", "dhcp"))
+			Expect(result.Subnets).To(HaveLen(0))
+
+			nc = podmanTest.Podman([]string{"network", "rm", net})
+			nc.WaitWithDefaultTimeout()
+			Expect(nc).Should(Exit(0))
+		})
+	}
+
+	It("podman network create/remove ipvlan as driver (-d) with device name", func() {
+		// Netavark currently does not support ipvlan
 		SkipIfNetavark(podmanTest)
-		net := "macvlan" + stringid.GenerateNonCryptoID()
-		nc := podmanTest.Podman([]string{"network", "create", "-d", "macvlan", "-o", "parent=lo", net})
+		net := "ipvlan" + stringid.GenerateRandomID()
+		nc := podmanTest.Podman([]string{"network", "create", "-d", "ipvlan", "-o", "parent=lo", net})
 		nc.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
 		Expect(nc).Should(Exit(0))
@@ -624,11 +615,11 @@ var _ = Describe("Podman network", func() {
 
 		var results []types.Network
 		err := json.Unmarshal([]byte(inspect.OutputToString()), &results)
-		Expect(err).To(BeNil())
+		Expect(err).ToNot(HaveOccurred())
 		Expect(results).To(HaveLen(1))
 		result := results[0]
 
-		Expect(result).To(HaveField("Driver", "macvlan"))
+		Expect(result).To(HaveField("Driver", "ipvlan"))
 		Expect(result).To(HaveField("NetworkInterface", "lo"))
 		Expect(result.IPAMOptions).To(HaveKeyWithValue("driver", "dhcp"))
 		Expect(result.Subnets).To(HaveLen(0))
@@ -639,7 +630,7 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network exists", func() {
-		net := "net" + stringid.GenerateNonCryptoID()
+		net := "net" + stringid.GenerateRandomID()
 		session := podmanTest.Podman([]string{"network", "create", net})
 		session.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
@@ -649,13 +640,13 @@ var _ = Describe("Podman network", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
-		session = podmanTest.Podman([]string{"network", "exists", stringid.GenerateNonCryptoID()})
+		session = podmanTest.Podman([]string{"network", "exists", stringid.GenerateRandomID()})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(1))
 	})
 
 	It("podman network create macvlan with network info and options", func() {
-		net := "macvlan" + stringid.GenerateNonCryptoID()
+		net := "macvlan" + stringid.GenerateRandomID()
 		nc := podmanTest.Podman([]string{"network", "create", "-d", "macvlan", "-o", "parent=lo", "-o", "mtu=1500", "--gateway", "192.168.1.254", "--subnet", "192.168.1.0/24", net})
 		nc.WaitWithDefaultTimeout()
 		defer podmanTest.removeNetwork(net)
@@ -667,7 +658,7 @@ var _ = Describe("Podman network", func() {
 
 		var results []types.Network
 		err := json.Unmarshal([]byte(inspect.OutputToString()), &results)
-		Expect(err).To(BeNil())
+		Expect(err).ToNot(HaveOccurred())
 		Expect(results).To(HaveLen(1))
 		result := results[0]
 
@@ -686,12 +677,8 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network prune --filter", func() {
-		// set custom cni directory to prevent flakes
-		podmanTest.NetworkConfigDir = tempdir
-		if IsRemote() {
-			podmanTest.RestartRemoteService()
-		}
-		net1 := "macvlan" + stringid.GenerateNonCryptoID() + "net1"
+		useCustomNetworkDir(podmanTest, tempdir)
+		net1 := "macvlan" + stringid.GenerateRandomID() + "net1"
 
 		nc := podmanTest.Podman([]string{"network", "create", net1})
 		nc.WaitWithDefaultTimeout()
@@ -734,17 +721,13 @@ var _ = Describe("Podman network", func() {
 	})
 
 	It("podman network prune", func() {
-		// set custom cni directory to prevent flakes
-		podmanTest.NetworkConfigDir = tempdir
-		if IsRemote() {
-			podmanTest.RestartRemoteService()
-		}
+		useCustomNetworkDir(podmanTest, tempdir)
 		// Create two networks
 		// Check they are there
 		// Run a container on one of them
 		// Network Prune
 		// Check that one has been pruned, other remains
-		net := "macvlan" + stringid.GenerateNonCryptoID()
+		net := "macvlan" + stringid.GenerateRandomID()
 		net1 := net + "1"
 		net2 := net + "2"
 		nc := podmanTest.Podman([]string{"network", "create", net1})
